//
//  All functions here should be OnX callbacks, that is to say
// functions with signature :
//   extern "C" {
//     void callback_without_arguments(Slash::UI::IUI&);
//     void callback_with_arguments(Slash::UI::IUI&,
//                                  const std::vector<std::string>&);
//   }
//

// Slash :
#include <Slash/Core/ISession.h>
#include <Slash/UI/IUI.h>

// Lib :
#include <Lib/System.h>
#include <Lib/Manager.h>
#include <Lib/Printer.h>
#include <Lib/Out.h>

// OnX :
#include <OnX/Helpers/OnX.h>
#include <OnX/Helpers/Inventor.h>

#include "Helpers_AIDA.h"

// G4Lab :
#include <G4Lab/Interfaces/IGeant4Manager.h>

// AIDA :
#include <AIDA/IPlotterFactory.h>
#include <AIDA/IPlotter.h>
#include <AIDA/IPlotterRegion.h>

extern "C" {

//////////////////////////////////////////////////////////////////////////////
/// G4LabTemplateOne/scripts/OnX/G4LabTemplateOne.onx ////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void G4LabTemplateOne_Initialize(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
// Should be executed (in a widget create callback) 
// when the GUI is constructed 
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::UI::ISoViewer* soViewer = ui_SoViewer(aUI);
  if(soViewer) {
    soViewer->setDecoration(true);
    soViewer->setViewing(true);
  }

  aUI.setCallback("OnX_File_exit","activate","DLD","G4Lab G4Lab_exit");

  Slash::Core::ISession& session = aUI.session();

  session.setParameter("modeling.highlightColor","white");
  session.setParameter("modeling.trajectories","immediate_all");
  session.setParameter("event.scan","yes");
  session.setParameter("g4Lab.nextEventAction","Trajectories_all_immediate");
    
  IGeant4Manager* g4Manager = 
    Lib_findManager(session,"Geant4Manager",IGeant4Manager);
  if(!g4Manager) {
    aUI.echo("G4LabTemplateOne_Initialize : Geant4Manager not found.");
  } else {
    // Declare a callback executed at each end of event :
    g4Manager->setEventEndScript("DLD","G4LabTemplateOne G4LabTemplateOne_eventEnd");
    //Declare G4Lab types (PV, Trajectory, HitsCollections, etc...).
    g4Manager->initialize(); 
  }    
  
  // Create some extra histos used in the callbacks :
  AIDA::IAnalysisFactory* aida = find_aida(session);
  if(aida) {
    AIDA::ITree* memoryTree = find_memoryTree(*aida);
    if(memoryTree) {
      AIDA::IHistogramFactory* histogramFactory = 
        aida->createHistogramFactory(*memoryTree);
      if(histogramFactory) {
        histogramFactory->createHistogram1D("h_e_E","e- E",100,0,20);
        histogramFactory->createHistogram1D("h_e_E_cut","e- E cut",100,0,20);
        delete histogramFactory;
      }
    }
  }

}
//////////////////////////////////////////////////////////////////////////////
void G4LabTemplateOne_Finalize(
 Slash::Core::ISession&
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
}
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void G4LabTemplateOne_InputNextEventAction_ok(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  std::string action;
  if(!aUI.parameterValue("G4LabTemplateOne_InputNextEventAction_input_action.value",action)) return;
  aUI.session().setParameter("g4Lab.nextEventAction",action);
}
//////////////////////////////////////////////////////////////////////////////
void G4LabTemplateOne_execute_eventEnd(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  std::string value;
  if(!session.parameterValue("event.scan",value)) return;
  // To avoid falling in the ui.steer();
  session.setParameter("event.scan","no");

  aUI.executeScript("DLD","G4LabTemplateOne G4LabTemplateOne_eventEnd");

  // Restore previous event.scan :
  session.setParameter("event.scan",value);
}
//////////////////////////////////////////////////////////////////////////////
/// scripts/OnX/Geant.onx ////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
void G4LabTemplateOne_Geant4_init(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  IGeant4Manager* g4Manager = 
    Lib_findManager(aUI.session(),"Geant4Manager",IGeant4Manager);
  if(!g4Manager) {
    Lib::Out out(aUI.session().printer());
    out << "G4LabTemplateOne_Geant4_init :"
        << " Geant4Manager not found." 
        << Lib::endl;
    return;
  }

  std::string file = Lib::System::getenv("G4LABTEMPLATEONEROOT")+"/scripts/Geant4/init.g4m";

  g4Manager->executeG4Script("/control/execute "+file);
}
//////////////////////////////////////////////////////////////////////////////
void G4LabTemplateOne_Geant4_detector(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();
  
  data_collect(session,"PV","name==\"BarrelPV\"");
  data_visualize(session);

  data_collect(session,"PV","name==\"LeftCapPV\"");
  data_visualize(session);

  data_collect(session,"PV","name==\"RightCapPV\"");
  data_visualize(session);

  //data_collect(session,"PV","name==\"CellPV\"");
  //data_visualize(session);

  Slash::UI::ISoViewer* soViewer = ui_SoViewer(aUI);
  if(soViewer) soViewer->viewAll();
}

//////////////////////////////////////////////////////////////////////////////
void G4LabTemplateOne_plot_histos(
 Slash::UI::IUI& aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Slash::Core::ISession& session = aUI.session();

  // Find AIDA :
  AIDA::IAnalysisFactory* aida = find_aida(session);
  if(!aida) return;

  // Find memory tree :
  AIDA::ITree* memoryTree = find_memoryTree(*aida);
  if(!memoryTree) return;

  AIDA::IPlotterFactory* plotterFactory = aida->createPlotterFactory();
  if(!plotterFactory) return;
  AIDA::IPlotter* plotter = plotterFactory->create("");
  if(!plotter) return;

  plotter->createRegions(1,3,0);

  {
    AIDA::IHistogram1D* h_e_E = find_histogram1D(*memoryTree,"h_e_E");
    if(h_e_E) {
      AIDA::IPlotterRegion& region = plotter->currentRegion();
      region.plot(*h_e_E);
    }
  }

  {
    AIDA::IHistogram1D* h_e_E_cut = find_histogram1D(*memoryTree,"h_e_E_cut");
    if(h_e_E_cut) {
      AIDA::IPlotterRegion& region = plotter->next();
      region.plot(*h_e_E_cut);
    }
  }

  {
    AIDA::IHistogram1D* histo_DE = find_histogram1D(*memoryTree,"histo_DE");
    if(histo_DE) {
      AIDA::IPlotterRegion& region = plotter->next();
      region.plot(*histo_DE);
    }
  }

  delete plotter;
  delete plotterFactory;

}
//////////////////////////////////////////////////////////////////////////////
void G4LabTemplateOne_debug(
 Slash::UI::IUI& //aUI
) 
//////////////////////////////////////////////////////////////////////////////
//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!//
{
  Lib::Printer printer;
  Lib::Out out(printer);
  out << "G4LabTemplateOne_debug" << Lib::endl;
}

} // extern "C"
